#!/usr/bin/python

# The MIT License (MIT)
#
# Copyright (c) 2014 Dave Parsons
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the 'Software'), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import struct
import sys


def ByteToHex( byteStr ):
    return ''.join( [ '%02X ' % ord( x ) for x in byteStr ] ).strip()


def dumpsmc(f, key):

    # vSMC Key Data Structure
    # Offset  Length  struct Type Description
    # ----------------------------------------
    # 0x00/00 0x04/04 4s     int  Key name (byte reversed e.g. #KEY is YEK#)
    # 0x04/04 0x01/01 B      byte Length of returned data
    # 0x05/05 0x04/04 4s     int  Data type (byte reversed e.g. ui32 is 23iu)
    # 0x09/09 0x01/01 B      byte Flag R/W
    # 0x0a/10 0x06/06 6x     byte Padding
    # 0x10/16 0x08/08 Q      ptr  Internal VMware routine
    # 0x18/24 0x30/48 48B    byte Data

    # Print vSMC offset
    print '#KEY offset: ' + hex(key)

    # Do Until OSK1 read
    i = 0
    while True:

        #  Read key into struct str and data byte str
        offset = key + (i * 72)
        f.seek(offset)
        smc_key = struct.unpack('=4sB4sB6xQ', f.read(24))
        smc_data = f.read(smc_key[1])

        #  Dump entry
        print str(i).zfill(3)  \
            + ' ' + smc_key[0][::-1] \
            + ' ' + str(smc_key[1]).zfill(2) \
            + ' ' + smc_key[2][::-1] \
            + ' ' + '{0:#0{1}x}'.format(smc_key[3],4) \
            + ' ' + hex(smc_key[4]) \
            + ' ' + ByteToHex(smc_data)

        # Simple chck for V0 or V1 vSMC
        if smc_key[0] == 'rdA$':
            smcV1 = True
        else:
            smcV1 = False

        # Exit when OSK1 has been read
        if smc_key[0] == '1KSO':
            break
        else:
            i += 1

    if smcV1:
        print '\nSMC V1 Table smc.version = 1'
    else:
        print '\nSMC V0 Table smc.version = 0'


def dumpvmx(name):
    with open(name, 'r+b') as f:
        # Patch file
        print 'Dump of: ' + name

        # Read file into string variable
        vmx = f.read()

        # Setup hex string for #KEY key
        key_key = '\x59\x45\x4B\x23\x04\x32\x33\x69\x75'

        # Find '#KEY' keys
        key = vmx.find(key_key)
        print '\nappleSMCTable 0\n'
        dumpsmc(f, key)

        key = vmx.rfind(key_key)
        print '\nappleSMCTable 1\n'
        dumpsmc(f, key)

        # Tidy up
        f.close()


def main():
    if len(sys.argv) >= 2:
        vmx_path = sys.argv[1]
    else:
        print 'Please pass file name!'
        return
    dumpvmx(vmx_path)

if __name__ == '__main__':
    main()
